@@ -1,23 +1,20 @@
 package AnyEvent::ProcessPool;
 # ABSTRACT: Asynchronously runs code concurrently in a pool of perl processes
-$AnyEvent::ProcessPool::VERSION = '0.06_001'; # TRIAL
+$AnyEvent::ProcessPool::VERSION = '0.01';
 
-$AnyEvent::ProcessPool::VERSION = '0.06001';use common::sense;
+use strict;
+use warnings;
 use Carp;
 use AnyEvent;
-use AnyEvent::Util;
 use AnyEvent::ProcessPool::Process;
-use AnyEvent::ProcessPool::Task;
-use AnyEvent::ProcessPool::Util qw(next_id cpu_count);
+use AnyEvent::ProcessPool::Util 'next_id';
 
 sub new {
   my ($class, %param) = @_;
 
   my $self = bless {
-    workers  => $param{workers} || cpu_count,
+    workers  => $param{workers} || croak 'expected parameter "workers"',
     limit    => $param{limit},
-    include  => $param{include},
-    pid      => $$,
     pool     => [], # array of AE::PP::Process objects
     queue    => [], # array of [id, code] tasks
     complete => {}, # task_id => condvar: signals result to caller
@@ -25,10 +22,6 @@ sub new {
   }, $class;
 
   # Initialize workers but do not yet wait for them to be started
-  if ($self->{limit}) {
-    $AnyEvent::Util::MAX_FORKS = $self->{limit};
-  }
-
   foreach (1 .. $self->{workers}) {
     my $worker = AnyEvent::ProcessPool::Process->new(limit => $self->{limit});
     push @{$self->{pool}}, $worker;
@@ -37,42 +30,29 @@ sub new {
   return $self;
 }
 
-sub join {
-  my $self = shift;
-  foreach my $task_id (keys %{$self->{complete}}) {
-    if (my $cv = $self->{complete}{$task_id}) {
-      $cv->recv;
-    }
-  }
-}
-
 sub DESTROY {
   my ($self, $global) = @_;
-  return unless $self;
-  return unless $self->{pid} == $$;
 
-  # Unblock watchers for any remaining pending tasks
-  if (ref $self->{pending}) {
-    foreach my $cv (values %{$self->{pending}}) {
-      $cv->croak('AnyEvent::ProcessPool destroyed with pending tasks remaining');
+  if ($self) {
+    # Unblock watchers for any remaining pending tasks
+    if (ref $self->{pending}) {
+      foreach my $cv (values %{$self->{pending}}) {
+        $cv->croak('AnyEvent::ProcessPool destroyed with pending tasks remaining');
+      }
     }
-  }
 
-  # Terminate any workers still alive
-  if (ref $self->{pool}) {
-    foreach my $worker (@{$self->{pool}}) {
-      $worker->stop if $worker;
+    # Terminate any workers still alive
+    if (ref $self->{pool}) {
+      $_->stop foreach @{$self->{pool}};
     }
   }
 }
 
 sub async {
-  my $self = shift;
-  my $code = shift;
-  my $id   = next_id;
-  my $task = AnyEvent::ProcessPool::Task->new($code, [@_]);
+  my ($self, $code) = @_;
+  my $id = next_id;
   $self->{complete}{$id} = AE::cv;
-  push @{$self->{queue}}, [$id, $task];
+  push @{$self->{queue}}, [$id, $code];
   $self->process_queue;
   return $self->{complete}{$id};
 }
@@ -82,25 +62,17 @@ sub process_queue {
   my $queue = $self->{queue};
   my $pool  = $self->{pool};
 
-  if (@$queue && @$pool) {
-    my ($id, $task) = @{shift @$queue};
+  while (@$queue && @$pool) {
+    my ($id, $code) = @{shift @$queue};
     my $worker = shift @$pool;
 
-    $self->{pending}{$id} = $worker->run($task);
+    $self->{pending}{$id} = $worker->run($code);
 
     $self->{pending}{$id}->cb(sub{
-      my $task = shift->recv;
-
-      if ($task->failed) {
-        $self->{complete}{$id}->croak($task->result);
-      } else {
-        $self->{complete}{$id}->send($task->result);
-      }
-
+      $self->{complete}{$id}->send(shift->recv);
       delete $self->{pending}{$id};
       delete $self->{complete}{$id};
-
-      push @{$self->{pool}}, $worker;
+      push @$pool, $worker;
       $self->process_queue;
     });
   }
@@ -120,7 +92,7 @@ AnyEvent::ProcessPool - Asynchronously runs code concurrently in a pool of perl
 
 =head1 VERSION
 
-version 0.06_001
+version 0.01
 
 =head1 SYNOPSIS
 
@@ -129,7 +101,6 @@ version 0.06_001
   my $pool = AnyEvent::ProcessPool->new(
     workers => 8,
     limit   => 10,
-    include => ['lib', 'some/lib/path'],
   );
 
   my $condvar = $pool->async(sub{
@@ -139,6 +110,10 @@ version 0.06_001
   # Block until result is ready
   my $result = $condvar->recv;
 
+  if ($result eq '...') {
+    ...
+  }
+
 =head1 DESCRIPTION
 
 Executes code using a pool a forked Perl subprocesses. Supports configurable
@@ -146,12 +121,12 @@ pool size, automatically restarting processes after a configurable number of
 requests, and closures (with the caveat that changes are not propagated back to
 the parent process).
 
-=head1 CONSTRUCTOR
+=head1 ATTRIBUTES
 
 =head2 workers
 
-Required attribute specifying the number of worker processes to launch.
-Defaults to the number of CPUs.
+Required attribute specifying the number of worker processes to launch. Must be
+a positive integer.
 
 =head2 limit
 
@@ -160,54 +135,14 @@ performing C<limit> tasks. This can be useful when calling code which may be
 leaky. When unspecified or set to zero, worker processes will only be restarted
 if it unexpectedly fails.
 
-=head2 include
-
-An optional array ref of paths to add to the perl command string used to start
-the sub-process worker.
-
 =head1 METHODS
 
 =head2 async
 
-Executes the supplied code ref in a worker sub-process. Remaining (optional)
-arguments are passed unchanged to the code ref in the worker process. Returns a
+Executes the supplied code ref in a worker sub-process. Returns a
 L<condvar|AnyEvent/CONDITION VARIABLES> that will block and return the task
 result when C<recv> is called on it.
 
-Alternately, the name of a task class may be supplied. The class must implement
-the methods 'new' (as a constructor) and 'run'. When using a task class, the
-arguments will be passed to the constructor (new) and the result of 'run' will
-be returned.
-
-  # With an anonymous subroutine
-  my $cv = $pool->async(sub{ ... });
-
-  # With a code ref
-  my $cv = $pool->async(\&do_stuff);
-
-  # With optional parameter list
-  my $cv = $pool->async(sub{ ... }, $arg1, $arg2, ...);
-
-  # With a task class
-  my $cv = $pool->async('My::Task', $arg1, ...);
-
-=head2 join
-
-Blocks until all pending tasks have completed. This does not prevent new tasks
-from being queued while waiting (for example, in the callback of an already
-queued task's condvar).
-
-=head1 PIPELINES
-
-Pipelinelines are alternative way of using the process pool. See
-L<AnyEvent::ProcessPool::Pipeline> for details.
-
-  use AnyEvent::ProcessPool::Pipeline;
-
-  pipeline workers => 4,
-    in  { get_next_task() }
-    out { do_stuff_with_result(shift->recv) };
-
 =head1 DIAGNOSTICS
 
 =head2 Task errors
@@ -216,14 +151,25 @@ Error messages resulting from a C<die> or C<croak> in task code executed in a
 worker process are rethrown in the parent process when the condition variable's
 C<recv> method is called.
 
-=head1 CAVEATS ON MSWIN32
+=head2 "AnyEvent::ProcessPool::Worker: ..." (warning)
 
-In addition to the usual caveats with regard to emulated forking on Windows,
-the platform's poor support for pipes means resorting to a pair of TCP sockets
-instead (which works but is much, much, slower). See the notes for
-C<portable_socketpair> and C<fork_call> in L<AnyEvent::Util>.
+When a worker sub-process emits output to C<STDERR>, the process pool warns
+the message out to its own C<STDERR>.
 
-=head1 SEE ALSO
+=head2 "error launching worker process: ..."
+
+Thrown when a worker sub-process failed to launch due to an execution error.
+
+=head2 "worker terminated in response to signal: ..."
+
+Thrown when a worker sub-process exits as a result of a signal received.
+
+=head2 "worker terminated with non-zero exit status: ..."
+
+Thrown when a worker sub-process terminates with a non-zero exit code. The
+worker will be automatically restarted.
+
+=head1 ALTERNATIVES
 
 =over
 
